"""func for yunet post process 
"""


from itertools import product

import cv2
import numpy as np


FACE_LANDMARK_COLORS = [
    (0, 255, 0),
    (0, 255, 0),
    (0, 0, 255),
    (20, 255, 255),
    (20, 255, 255),
]

SCORE_THRESHOLD = 0.97

NMS_SCORE = 0.3

TOP_K = 50


VARIANCE = np.array([0.1, 0.2], dtype=np.float32)

MIN_STEPS_LIST = [[10, 16, 24], [32, 48], [64, 96], [128, 192, 256]]

STEPS = [8, 16, 32, 64]


def decodeRawFaceDataList(
    inputSize,
    confs,
    ious,
    locs,
    priors,
    var = VARIANCE,
):
    # get score
    cls_scores = confs[:, 1]
    # clamp
    ious = np.clip(ious, 0, 1)
    scores = np.sqrt(cls_scores * ious)
    scores = scores[:, np.newaxis]

    # get bboxes
    frSize = np.array(inputSize).astype(np.int32)
    bboxes = np.hstack((
        (priors[:, 0:2] + locs[:, 0:2] * var[0] * priors[:, 2:4]) * frSize,
        (priors[:, 2:4] * np.exp(locs[:, 2:4] * var)) * frSize
    )).astype(np.float32)

    # (x_c, y_c, w, h) -> (x1, y1, w, h)
    bboxes[:, 0:2] -= bboxes[:, 2:4] / 2

    # get landmarks
    landmarks = np.hstack((
        (priors[:, 0:2] + locs[:,  4: 6] * var[0] * priors[:, 2:4]) * frSize,
        (priors[:, 0:2] + locs[:,  6: 8] * var[0] * priors[:, 2:4]) * frSize,
        (priors[:, 0:2] + locs[:,  8:10] * var[0] * priors[:, 2:4]) * frSize,
        (priors[:, 0:2] + locs[:, 10:12] * var[0] * priors[:, 2:4]) * frSize,
        (priors[:, 0:2] + locs[:, 12:14] * var[0] * priors[:, 2:4]) * frSize
    ))

    dets = np.hstack((bboxes, landmarks, scores)).astype(np.float32)

    return dets


def getPriors(
    inputSize,
    min_steps_list=MIN_STEPS_LIST,
    steps=STEPS
):
    w, h = inputSize
    feature_map_2th = [((h + 1) // 2) // 2,
                       ((w + 1) // 2) // 2]
    feature_map_3th = [feature_map_2th[0] // 2,
                       feature_map_2th[1] // 2]
    feature_map_4th = [feature_map_3th[0] // 2,
                       feature_map_3th[1] // 2]
    feature_map_5th = [feature_map_4th[0] // 2,
                       feature_map_4th[1] // 2]
    feature_map_6th = [feature_map_5th[0] // 2,
                       feature_map_5th[1] // 2]

    feature_maps = [feature_map_3th, feature_map_4th,
                    feature_map_5th, feature_map_6th]

    priors = []
    for k, f in enumerate(feature_maps):
        min_sizes = min_steps_list[k]
        for i, j in product(range(f[0]), range(f[1])):  # i->h, j->w
            for min_size in min_sizes:
                s_kx = min_size / w
                s_ky = min_size / h

                cx = (j + 0.5) * steps[k] / w
                cy = (i + 0.5) * steps[k] / h

                priors.append([cx, cy, s_kx, s_ky])
    # print("Priors length =", len(priors))
    priors = np.array(priors, dtype=np.float32)
    return priors


def getOutputSize(inputSize):
    output_1 = tuple(i//2//2//2 for i in inputSize)
    output_2 = tuple(i//2 for i in output_1)
    output_3 = tuple(i//2 for i in output_2)
    output_4 = tuple(i//2 for i in output_3)
    return output_1, output_2, output_3, output_4
    

def nms(
    decoded_dets,
    score_threshold=SCORE_THRESHOLD,
    nms_threshold=NMS_SCORE,
    top_k=TOP_K
):
    kept_idx = cv2.dnn.NMSBoxes(
        bboxes=decoded_dets[:, 0:4].tolist(),
        scores=decoded_dets[:, -1].tolist(),
        score_threshold=score_threshold,
        nms_threshold=nms_threshold,
        top_k=top_k
    )

    if len(kept_idx) > 0:
        nms_dets = decoded_dets[kept_idx]
        if len(nms_dets.shape) > 2:
            nms_dets = np.squeeze(nms_dets, axis=1)
    else:
        nms_dets = np.empty(shape=(0, 15), dtype=np.float32)

    return nms_dets

def logsumexp(a, axis=None, b=None, keepdims=False, return_sign=False):
    """Compute the log of the sum of exponentials of input elements.

    Parameters
    ----------
    a : array_like
        Input array.
    axis : None or int or tuple of ints, optional
        Axis or axes over which the sum is taken. By default `axis` is None,
        and all elements are summed.

        .. versionadded:: 0.11.0
    keepdims : bool, optional
        If this is set to True, the axes which are reduced are left in the
        result as dimensions with size one. With this option, the result
        will broadcast correctly against the original array.

        .. versionadded:: 0.15.0
    b : array-like, optional
        Scaling factor for exp(`a`) must be of the same shape as `a` or
        broadcastable to `a`. These values may be negative in order to
        implement subtraction.

        .. versionadded:: 0.12.0
    return_sign : bool, optional
        If this is set to True, the result will be a pair containing sign
        information; if False, results that are negative will be returned
        as NaN. Default is False (no sign information).

        .. versionadded:: 0.16.0

    Returns
    -------
    res : ndarray
        The result, ``np.log(np.sum(np.exp(a)))`` calculated in a numerically
        more stable way. If `b` is given then ``np.log(np.sum(b*np.exp(a)))``
        is returned.
    sgn : ndarray
        If return_sign is True, this will be an array of floating-point
        numbers matching res and +1, 0, or -1 depending on the sign
        of the result. If False, only one result is returned.

    See Also
    --------
    numpy.logaddexp, numpy.logaddexp2

    Notes
    -----
    NumPy has a logaddexp function which is very similar to `logsumexp`, but
    only handles two arguments. `logaddexp.reduce` is similar to this
    function, but may be less stable.

    Examples
    --------
    >>> from scipy.special import logsumexp
    >>> a = np.arange(10)
    >>> np.log(np.sum(np.exp(a)))
    9.4586297444267107
    >>> logsumexp(a)
    9.4586297444267107

    With weights

    >>> a = np.arange(10)
    >>> b = np.arange(10, 0, -1)
    >>> logsumexp(a, b=b)
    9.9170178533034665
    >>> np.log(np.sum(b*np.exp(a)))
    9.9170178533034647

    Returning a sign flag

    >>> logsumexp([1,2],b=[1,-1],return_sign=True)
    (1.5413248546129181, -1.0)

    Notice that `logsumexp` does not directly support masked arrays. To use it
    on a masked array, convert the mask into zero weights:

    >>> a = np.ma.array([np.log(2), 2, np.log(3)],
    ...                  mask=[False, True, False])
    >>> b = (~a.mask).astype(int)
    >>> logsumexp(a.data, b=b), np.log(5)
    1.6094379124341005, 1.6094379124341005

    """
    if b is not None:
        a, b = np.broadcast_arrays(a, b)
        if np.any(b == 0):
            a = a + 0.  # promote to at least float
            a[b == 0] = -np.inf

    a_max = np.amax(a, axis=axis, keepdims=True)

    if a_max.ndim > 0:
        a_max[~np.isfinite(a_max)] = 0
    elif not np.isfinite(a_max):
        a_max = 0

    if b is not None:
        b = np.asarray(b)
        tmp = b * np.exp(a - a_max)
    else:
        tmp = np.exp(a - a_max)

    # suppress warnings about log of zero
    with np.errstate(divide='ignore'):
        s = np.sum(tmp, axis=axis, keepdims=keepdims)
        if return_sign:
            sgn = np.sign(s)
            s *= sgn  # /= makes more sense but we need zero -> zero
        out = np.log(s)

    if not keepdims:
        a_max = np.squeeze(a_max, axis=axis)
    out += a_max

    if return_sign:
        return out, sgn
    else:
        return out

def softmax(x, axis=None):
    r"""
    Softmax function

    The softmax function transforms each element of a collection by
    computing the exponential of each element divided by the sum of the
    exponentials of all the elements. That is, if `x` is a one-dimensional
    numpy array::

        softmax(x) = np.exp(x)/sum(np.exp(x))

    Parameters
    ----------
    x : array_like
        Input array.
    axis : int or tuple of ints, optional
        Axis to compute values along. Default is None and softmax will be
        computed over the entire array `x`.

    Returns
    -------
    s : ndarray
        An array the same shape as `x`. The result will sum to 1 along the
        specified axis.

    Notes
    -----
    The formula for the softmax function :math:`\sigma(x)` for a vector
    :math:`x = \{x_0, x_1, ..., x_{n-1}\}` is

    .. math:: \sigma(x)_j = \frac{e^{x_j}}{\sum_k e^{x_k}}

    The `softmax` function is the gradient of `logsumexp`.

    .. versionadded:: 1.2.0

    Examples
    --------
    >>> from scipy.special import softmax
    >>> np.set_printoptions(precision=5)

    >>> x = np.array([[1, 0.5, 0.2, 3],
    ...               [1,  -1,   7, 3],
    ...               [2,  12,  13, 3]])
    ...

    Compute the softmax transformation over the entire array.

    >>> m = softmax(x)
    >>> m
    array([[  4.48309e-06,   2.71913e-06,   2.01438e-06,   3.31258e-05],
           [  4.48309e-06,   6.06720e-07,   1.80861e-03,   3.31258e-05],
           [  1.21863e-05,   2.68421e-01,   7.29644e-01,   3.31258e-05]])

    >>> m.sum()
    1.0000000000000002

    Compute the softmax transformation along the first axis (i.e., the
    columns).

    >>> m = softmax(x, axis=0)

    >>> m
    array([[  2.11942e-01,   1.01300e-05,   2.75394e-06,   3.33333e-01],
           [  2.11942e-01,   2.26030e-06,   2.47262e-03,   3.33333e-01],
           [  5.76117e-01,   9.99988e-01,   9.97525e-01,   3.33333e-01]])

    >>> m.sum(axis=0)
    array([ 1.,  1.,  1.,  1.])

    Compute the softmax transformation along the second axis (i.e., the rows).

    >>> m = softmax(x, axis=1)
    >>> m
    array([[  1.05877e-01,   6.42177e-02,   4.75736e-02,   7.82332e-01],
           [  2.42746e-03,   3.28521e-04,   9.79307e-01,   1.79366e-02],
           [  1.22094e-05,   2.68929e-01,   7.31025e-01,   3.31885e-05]])

    >>> m.sum(axis=1)
    array([ 1.,  1.,  1.])

    """

    # compute in log space for numerical stability
    return np.exp(x - logsumexp(x, axis=axis, keepdims=True))
